#include <qpixmap.h>
#include <qpainter.h>
#include <qfontmetrics.h>

#include "math.h"

#include "osdframe.moc"
#include "osdframe.h"



OSDFrame::OSDFrame(int size, QColor f, QColor b, float t,
		   int eff, QWidget *parent):
  QFrame(parent, 0, 
	 WType_TopLevel| WStyle_NoBorder | WX11BypassWM | WStyle_StaysOnTop)
{
  fgColor = f;
  bgColor = b;

  frameRoot = new KRootPixmap(this);

  titleFont = QFont("Helvetica", size);

  resize(320, 20);

  frameRoot->setFadeEffect(t, bgColor);

  frameRoot->start();

  paintTimer = new QTimer(this);
  connect(paintTimer, SIGNAL(timeout()), SLOT(paint()));
  
  effect = eff;

  title = QString("this is the song title");

  QFontMetrics fm(titleFont);

  width = fm.width(title) + 20;
  height = fm.height();

  resize(width, height + 2);

  if (effect == OSD_TEST) {
    paintTimer->start(50);
  }
}

OSDFrame::~OSDFrame() {

  paintTimer->stop();
  frameRoot->stop();
}

void OSDFrame::setFontSize(int size) {

  titleFont = QFont("Helvetica", size);
}

void OSDFrame::start(QString t) {

  QFontMetrics fm(titleFont);

  width = fm.width(t) + 20;
  height = fm.height();

  resize(width, height + 2);

  effect = FADE_IN;
  title = t;

  i = M_PI / 2;
  wait = 0;

  paintTimer->start(30);
}

void OSDFrame::paintEvent(QPaintEvent *e) {


}

void OSDFrame::paint() {

  int xPos = 0;
  int yPos = 0;

  if (effect == OSD_TEST) {
    
    xPos = 10;
    yPos = (int) height - 5;
  }

  if (effect == FADE_IN) {

    if (i <= M_PI) i += 0.05;
    if (i > M_PI) {
      i = M_PI;
      effect = FADE_WAIT;      
    }

    xPos = (int) (-sin(i) * width + 10);
    yPos = (int) height - 5;
  }

  else if (effect == FADE_WAIT) {

    wait++;
    if (wait > 80)
      effect = FADE_OUT;

    xPos = (int) (-sin(i) * width + 10);
    yPos = (int) height - 5;
  }


  else if (effect == FADE_OUT) {
    
    wait = 0;
    
    if (i <= 3 * M_PI / 2) i += 0.05;
    if (i > 3 * M_PI / 2) {
      i = 3 * M_PI / 2;
      effect = FADE_STOP;
    }

    xPos = (int) (sin(i) * width + 10);
    yPos = (int) height - 5;

  }

  QPixmap pmt(width, height);
  pmt.fill(this, width, height);

  QPainter pt;
  pt.begin(&pmt);
  pt.setFont(titleFont);

  pt.setPen(fgColor);
  pt.drawText(xPos, yPos, title);

  QPainter hvpainter(this);
  hvpainter.drawPixmap(0, 0, pmt);

  if (!isVisible())
    show();

  if (effect == FADE_STOP) {
    paintTimer->stop();
    hide();
  }
}
